/*  Copyright (C) 2000, 2006  Marc Toussaint (mtoussai@inf.ed.ac.uk)
    under the terms of the GNU LGPL (http://www.gnu.org/copyleft/lesser.html)
    see the `std.h' file for a full copyright statement  */

/*  This implements the online LWPR regression algorithm as described in:
    Sethu Vijayakumar, Aaron D'Souza and Stefan Schaal, "Incremental Online
    Learning in High Dimensions", Neural Computation, 17:12, pp. 2602-2632  (2005)
    (Please cite when using code) */

#include<lwpr.h>

char* HELP = "\
----------\n\
LWPR regression\n\
\n\
This executable allows to use LWPR from command line,\n\
reading and writing data files. The config file MT.cfg\n\
controlls all options, parameters, filenames, etc. Please\n\
refer to this file for further details.\n\
\n\
The LWPR regression algorithm as described in:\n\
Sethu Vijayakumar, Aaron D'Souza and Stefan Schaal,\n\
\"Incremental Online Learning in High Dimensions\",\n\
Neural Computation, 17:12, pp. 2602-2632  (2005)\n\
(Please cite when using code)\n\
\n\
Copyright (C) 2000, 2006  Marc Toussaint (mtoussai@inf.ed.ac.uk)\n\
----------\n";

void run(){
  Parameter<String> trainfile ("trainingDataFile","");
  Parameter<String> testfile  ("testDataFile","");
  Parameter<uint>   nCols     ("numberOfColumns");
  Parameter<uintA>  inputCols ("inputColumns");
  Parameter<uintA>  outputCols("outputColumns");
  Parameter<bool>   sdvNorm   ("useStdDevNorm");
  Parameter<uint>   nIter     ("numberOfIterations");
  Parameter<bool>   loadModel ("loadModel");
  Parameter<bool>   random    ("random");            
  Parameter<String> modelFile ("modelFile");
  Parameter<uint>   evalTime  ("evalTime");

  doubleA D,X,Y,Z;
 
  Lwpr l;
  if(sdvNorm) getStdDeviations(l.norm,X);
  if(loadModel) l.load(modelFile());
  
  if(trainfile().N()){
    D.readRaw(trainfile().ptr(),nCols);
    X=D.sub(0,-1,inputCols);
    Y=D.sub(0,-1,outputCols);
	  
    for(uint k=0;k<nIter;k++){
      if(random){
	MT::Permutation pi;
	pi.random(X.d0);
	for(int i=0;i<X.d0;i++) l.learn(X[pi(i)],Y[pi(i)]);
      }else{
	l.learn(X,Y);
      }
      double avg = sum(Y)/Y.d0;
      double var = sumOfSqr((Y-avg))/Y.d0;
      if(k%evalTime == 0){
	l.predict(X,Z);
	double mse = sqrDistance(Z,Y)/(double)Y.d0;
			  
	std::cout <<"iter =" << k <<" #rfs =" <<l.rfsno << " MSE =" 
		  << mse << " nMSE =" << mse/var << std::endl;
      }
    }
    l.report(std::cout);
	  
    l.predict(X,Z);
    write(X,Z,trainfile()+".out");
    std::cout <<"MSE on training data = " <<sqrDistance(Z,Y)/(double)Y.d0 <<std::endl;
    l.save(modelFile());
  }
 
  if(!trainfile().N() && !loadModel && testfile().N())
    HALT("please load model or train model before testing");
  if(testfile().N()){
    D.readRaw(testfile().ptr(),nCols);
    X=D.sub(0,-1,inputCols);
    Y=D.sub(0,-1,outputCols);
	  
    l.predict(X,Z);

    write(X,Z,testfile()+".out");
    std::cout <<"MSE on test data = " <<sqrDistance(Z,Y)/(double)Y.d0 <<std::endl;
  }
}

int main(int argn,char** argv){
  //in case you want to specify parameters on command line
  MT::init(argn,argv);

  std::cout <<HELP;

  run();

  return 0;
}
