/*  Copyright (C) 2000, 2006  Marc Toussaint (mtoussai@inf.ed.ac.uk)
    under the terms of the GNU LGPL (http://www.gnu.org/copyleft/lesser.html)
    see the `std.h' file for a full copyright statement  */

/*  LWPR is a regresion technique mainly developed by Sethu
    Vijayakumar. This file tests an implementation of this algorithm,
    coded by Narayanan Edakunni and Giorgos Petkos, cleaned up and
    made more efficient by myself. */

#include<lwpr.h>

void testLWPR(){
  uint i,k;

  //construct data
  doubleA X,Y;
#if 0  //1D data
  grid(X,1,-1.,1.,1000);
  Y.resize(X.d0,1);
  for(i=0;i<X.d0;i++) Y(i,0)=::sin(5.*X(i,0));
  MT::save(X,"grid");
#else  //2D data
  grid(X,2,-1.,1.,30);
  Y.resize(X.d0,1);
  for(i=0;i<X.d0;i++) Y(i,0)=::sin(5.*X(i,0)) * ::sin(7.*X(i,1));
#endif

  //NOTE: to inferface with your code, you could simply let
  //      X & Y refer to ordinary C-arrays: e.g.,
  //      X.referTo(p,samples*dimension); X.resize(samples,dimension);

  //mess up the input data normalization
  X*=2.;
  X+=1.;

  //save data
  MT::IOraw=true;
  std::ofstream da("data");
  for(i=0;i<X.d0;i++) da <<X[i] <<Y[i] <<std::endl; //full data
  if(X.d1==2){ Y.reshape(31,31); MT::save(Y,"dataY"); Y.reshape(X.d0,1); } //easier to display 3D

  //instantiate
  Lwpr l; //also reads parameters from config file or command line
  l.verbosity = 1; //outputs information during learning (default anyway)
  getStdDeviations(l.norm,X); //this is optional, a crude way to normalize the input data

  //NOTE: here, you could also change parameter settings explicitly
  //      if you don't like config files or command line: e.g.,
  //      l.initD=.01;

  //learn
  for(k=0;k<3;k++){ //cycles 3 times through the whole batch of data
    l.learn(X,Y);
    std::cout <<"iteration " <<k <<" #rfs=" <<l.rfsno <<std::endl;
  }

  //give a simple report
  l.report(std::cout);

  //if you like, enlarge the input domain
  //(to test how it behaves outside regions of observed data)
  //grid(X,1,-2.,4.,1000);

  //apply the learned mapping
  doubleA Z;
  l.predict(X,Z);

  //MSE:
  std::cout <<"MSE = " <<sqrDistance(Z,Y)/(double)Y.d0 <<std::endl;

  //save the output and the confidence bounds
  std::ofstream os("out");
  for(i=0;i<X.d0;i++) os <<X[i] <<Z[i] <<' ' <<Z[i]+l.confidence(X[i])<<' ' <<Z[i]-l.confidence(X[i]) <<std::endl;
  if(X.d1==2){ Z.reshape(31,31); MT::save(Z,"outY"); Z.reshape(X.d0,1); }

  //save the full model
  l.save("model");

  //reload model and test it
  Lwpr l2;
  l2.load("model");
  doubleA Z2;
  l2.predict(X,Z2);
  std::cout <<"maximum difference in output between original and reloaded = "
    <<absMax(Z-Z2) <<std::endl;

  //call gnuplot to see an output
  std::cout <<"trying to display output..." <<std::endl;
  if(X.d1==1) gnuplot("plot 'data' title 'true','out' us 1:2 title 'learned mean','out' us 1:3 title 'upper bound','out' us 1:4 title 'lower bound'");
  else gnuplot("splot 'dataY' matrix title 'true','outY' matrix title 'learned'");
}

int main(int argn,char** argv){
  //in case you want to specify parameters on command line
  MT::init(argn,argv);

  testLWPR();

  return 0;
}
